#----------------------------------------------------------------------------
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/system/Arch.pm,v 1.11 2008/10/06 05:09:33 aatre Exp $
#
# Veritas Software Corporation
#
# Package Name: Arch
#
# $Author: aatre $
# $Date: 2008/10/06 05:09:33 $
# $Revision: 1.11 $
#
# Description: This module provides the functionality which is OS dependant, 
#              allowing the agent FW to be supported on different OS 
#              types more easily.
#
# Authors: 011787
#----------------------------------------------------------------------------

#---------------------
# Version calls.
#---------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/system/Arch.pm,v 1.11 2008/10/06 05:09:33 aatre Exp $::ETVHeader");

#-------------------------
# Package declaration.
#-------------------------
package Arch;

#-------------------------------
# Include all packages required.
#-------------------------------
use LogInt;
use strict;

$::gbArchInitialised = 0; #set to uninitialised to start
$::giArchOSType      = 0; #set to uninitialised to start

#---------------------------------------------------------------
# constants that are available for multi-platform support.
#---------------------------------------------------------------
use constant OS_CONSOLE          => 1;
use constant OS_SYSTEM_LOG_FILE  => 2;

#-----------------
# sun lookups.
#-----------------
my %hFlagToString_SunOS = ( Arch::OS_CONSOLE         => "/dev/console",
                            Arch::OS_SYSTEM_LOG_FILE => "/var/adm/messages" );

my %hFlagToString_AIX   = ( Arch::OS_CONSOLE          =>  "/dev/console",
                            Arch::OS_SYSTEM_LOG_FILE  =>  "/var/adm/messages" );

my %hFlagToString_LINUX = ( Arch::OS_CONSOLE         => "/dev/console",
                            Arch::OS_SYSTEM_LOG_FILE => "/var/log/messages" );

my %hFlagToString_HPUX  = ( Arch::OS_CONSOLE         => "/dev/console",
                            Arch::OS_SYSTEM_LOG_FILE => "/var/adm/syslog" );

#-------------------------------------------
# Enumerate the OS's we do/will support.
#-------------------------------------------
use constant UNKNOWN_OS  => 101;
use constant SUN_OS      => 102;
use constant HPUX_OS     => 103;
use constant LINUX_OS    => 104;
use constant AIX_OS      => 105;


=pod

-----------------------------------------------------------------------------
Function: Boolean GetOSDependantSetting ( IN Integer VarName, 
                                          OUT String Setting )

Description:
  Gives the OS dependent setting for Console and Messages.

Parameters:
  VarName : OS dependent variable name. It can be one of the following:
                          OS_CONSOLE
                          OS_SYSTEM_LOG_FILE
  Setting : OS dependent setting for given variable.
	
Return Value:
  Boolean value in the set of {0, 1} where
        0 - false
        1 - true
-----------------------------------------------------------------------------

=cut

sub GetOSDependantSetting {

  my ( $iVarName,
       $rsSetting ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10134,
                   "Arch",
                   "GetOSDependantSetting",
                   "Subroutine %s called with: VarName [%s]",
                   $sMyName, $iVarName );

  my $iOSType = GetOSType();

  my $rhFlagToStringLookupHashRef = 0;

  if(Arch::SUN_OS == $iOSType){
    $rhFlagToStringLookupHashRef = \%hFlagToString_SunOS;
  } 
  elsif(Arch::AIX_OS == $iOSType) {
    $rhFlagToStringLookupHashRef = \%hFlagToString_AIX;
  } 
  elsif(Arch::LINUX_OS == $iOSType ) {
    $rhFlagToStringLookupHashRef = \%hFlagToString_LINUX;
  } 
  elsif(Arch::HPUX_OS == $iOSType ) {
    $rhFlagToStringLookupHashRef = \%hFlagToString_HPUX;
  } 
  else {
    LogInt::Message( LogInt::LLTrace,
                     10135,
                     "Arch",
                     "GetOSDependantSetting",
                     "UNKNOWN OS type" );
  }

  my $bLookupSuceeded = 0;

  if($rhFlagToStringLookupHashRef != 0) {
    if(exists $$rhFlagToStringLookupHashRef{$iVarName}) {
      $$rsSetting      = $$rhFlagToStringLookupHashRef{$iVarName};
      $bLookupSuceeded = 1; #set true
    }
    else {
      LogInt::Message( LogInt::LLTrace,
                       10136,
                       "Arch",
                       "GetOSDependantSetting",
                       "OS type doesnot exists" );
    }
  }
  else {
    LogInt::Message( LogInt::LLTrace,
                     10137,
                     "Arch",
                     "GetOSDependantSetting",
                     "OS type is not known" );
  }

  return $bLookupSuceeded;
}


=pod

-----------------------------------------------------------------------------
Function: Integer GetOSType( Void )

Description:
  Gives the Operating System type.

Parameters:
  Void : No parameters are required.

Return Value:
  Integer : Indicating OS type. It can be of the following.
                       UNKNOWN_OS    => 101;
                       SUN_OS        => 102;
                       HPUX_OS       => 103;
                       LINUX_OS      => 104;
                       AIX_OS        => 105;
-----------------------------------------------------------------------------

=cut

sub GetOSType {

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10138,
                   "Arch",
                   "GetOSType",
                   "Subroutine %s is called", $sMyName );
  
  if( $::gbArchInitialised ) {
    return $::giArchOSType; #already worked it out
  }

  #----------------------------------------
  # make sure we remember we've been here.
  #----------------------------------------
  $::gbArchInitialised = 1;

  #----------------------------------------------------------------
  # call uname -s and use string to determine OS
  # might need to look in multiple places to find 'uname'
  # on sun/solaris it should be at '/usr/bin/uname'
  #----------------------------------------------------------------

  #--------------------------------------
  # add more uname places as needed.
  #--------------------------------------
  my @lsPlacesToLookForUname = qw( /usr/bin/uname /bin/uname );

  my $sUnameResultString = "UNKNOWN";

  #-----------------------------------------------------------
  # spin the list of uname places - see which one we can call.
  #-----------------------------------------------------------
  foreach my $sPlaceToCheckForUname (@lsPlacesToLookForUname) {
    if(-x $sPlaceToCheckForUname) {
      #-----------------------------
      # found an exec - call it.
      #-----------------------------
      $sUnameResultString = `$sPlaceToCheckForUname -s`;

      #-------------------------------
      # chomp the returned string.
      #-------------------------------
      chomp $sUnameResultString;

      my $iReturnCode = $? >> 8;

      if( $iReturnCode != 0 ) {
        LogInt::Message( LogInt::LLError,
                         10139,
                         "Arch",
                         "GetOSType",
                         "Failed to get the OS type" );
      }   
      last; 
    }
  } 

  #---------------------------------------------------------
  # use $sUnameResultString in hash lookup
  # **POTENTIALLY** only the 'SunOS' one is correct
  #--------------------------------------------------------- 
  my %shUnameStringToOSTypeLookup = ("SunOS"    =>  Arch::SUN_OS,
                                     "HP-UX"    =>  Arch::HPUX_OS,
                                     "AIX"      =>  Arch::AIX_OS,
                                     "Linux"    =>  Arch::LINUX_OS,
                                     );

  if(exists $shUnameStringToOSTypeLookup{ $sUnameResultString }) {
    $::giArchOSType = $shUnameStringToOSTypeLookup{ $sUnameResultString };
  } 
  else {
    $::giArchOSType = Arch::UNKNOWN_OS;
  }

  return $::giArchOSType;
}


1; 

